#include <stdio.h>
#include "main.h"
#include "map.h"
#include "screen.h"


map_t *map_load_map(const char *fname)
{
    int    row, col, n_rows, n_cols, max_cols;
    char   c;
    FILE  *fp;
    map_t *map;

    if (!(fp = fopen(fname, "r")))
      ERR_FAIL("Could not open map file '%s'", fname);

    /* Count rows and cols */
    n_rows = n_cols = 0;
    max_cols = -1;
    while ((c = fgetc(fp)) != EOF)
    {
        if (c=='\r' || c=='\n')
        {
            ++n_rows;

            if (n_cols > max_cols)
              max_cols = n_cols;

            n_cols = 0;
        }
        else
          ++n_cols;
    }

    n_cols = max_cols;

    /* Create the map */
    map = calloc(1, sizeof(map_t));
    map->map = calloc(n_rows, sizeof(char *));
    for (row=0; row<n_rows; ++row)
      map->map[row] = calloc(n_cols, 1);
    map->n_rows = n_rows;
    map->n_cols = n_cols;

    /* Load the map */
    row = col = 0;
    fseek(fp, 0, SEEK_SET);
    while ((c = fgetc(fp)) != EOF)
    {
        /* Start a new row */
        if (c=='\r' || c=='\n')
        {
            ++row;
            col = 0;
        }

        /* Start point */
        else if (c == MAP_START)
        {
            map->start.row = row;
            map->start.col = col;
            map->map[row][col] = MAP_PATH;
            ++col;
        }
        
        /* Column entry */
        else
        {
            map->map[row][col] = c;
            ++col;
        }
    }

    fclose(fp);
    return map;
}


void map_free_map(map_t *map)
{
    int i;

    for (i=0; i<map->n_rows; ++i)
      free(map->map[i]);

    free(map);
}


void map_adjust_position(pos_t *pos, direction_t dir)
{
    switch (dir)
    {
        case UP: --pos->row; return;
        case DOWN: ++pos->row; return;
        case LEFT: --pos->col; return;
        case RIGHT: ++pos->col; return;
        default: return;
    }
}


void map_restore_position(const map_t *map, const pos_t *pos)
{
    screen_render_char(map->map[pos->row][pos->col], pos);
}


int map_positions_equal(const pos_t *pos1, const pos_t *pos2)
{
    return ((pos1->row == pos2->row) && (pos1->col == pos2->col));
}


void map_place_random(const map_t *map, const pos_t *pos)
{
    int   attempt;
    char  something;
    pos_t new_pos;

    something = map->map[pos->row][pos->col];

    /* 10 tries to find an empty spot to place 'something' */
    for (attempt=0; attempt<10; ++attempt)
    {
        new_pos.row = rand() % map->n_rows;
        new_pos.col = rand() % map->n_cols;

        /* Ooooops not an empty spot */
        if (map->map[new_pos.row][new_pos.col] != MAP_PATH)
          continue;
        
        /* Erase old spot and place something in the new one */
        map->map[pos->row][pos->col] = MAP_PATH;
        map->map[new_pos.row][new_pos.col] = something;
        screen_render_char(something, &new_pos);
        break;
    }
}
