#ifndef _MAP_H
#define _MAP_H

/*
 * The following is how things are represented on the map:
 *
 * Player: Player is never on the map, only the 'S' represents
 * the starting position.  When a map is loaded the 'S' is replaced
 * with the player.  Players has its own collision detection to see if
 * a passed position touches the player.
 *
 * Enemies: Enemies are on the map, which makes for easy collision detection:
 *          IS_ENEMY(map[x][y])
 *
 * Projectiles: Projectiles are never on the map, they are temporary.
 */


/* 2D position */
typedef struct 
{
    int row;
    int col;
} pos_t;


/* A map is just a 2-dimensional array of ascii */
typedef struct
{
    int     n_rows;
    int     n_cols;
    pos_t   start;  
    char  **map;
} map_t;


/* Things in the map (These make up the map files) */
#define MAP_WALL         '*'
#define MAP_START        'S'
#define MAP_PATH         ' '
#define MAP_POWER        'V'
#define MAP_ENEMY_TURRET '#'
#define MAP_ENEMY_MOVER  'H'


/* Test if the type is one of multiple enemy types */
#define IS_ENEMY(_t) (((_t) == MAP_ENEMY_TURRET) || ((_t) == MAP_ENEMY_MOVER))


/* Movement positions */
typedef enum {UP, DOWN, LEFT, RIGHT, NOWHERE, N_DIRECTIONS} direction_t;


/* Load a map from a file name (call map_free_map() when done) */
extern map_t *map_load_map(const char *fname);


/* Release resource from the map */
extern void map_free_map(map_t *map);


/* Move to a desired position */
extern void map_adjust_position(pos_t *pos, direction_t dir);


/* Restore position on the map */
extern void map_restore_position(const map_t *map, const pos_t *pos);


/* Returns true if the positions are identical */
extern int map_positions_equal(const pos_t *pos1, const pos_t *pos2);


/* Place the something at 'pos' pseudo-randomly somewhere */
extern void map_place_random(const map_t *map, const pos_t *pos);


#endif /* _MAP_H */
