#ifndef _PLAYER_H
#define _PLAYER_H

#include "main.h"
#include "map.h"


/* Our own local representation of shoot_t (enemy.h has one too) */
typedef struct
{
    int         is_shooting;
    pos_t       pos;
    char        symbol;
    direction_t dir;
} player_shoot_t;


/* One segment of what our player image looks like */
typedef struct
{
    pos_t pos;
    char  symbol;
} icon_t;


/* The player */
#define PLAYER_MAX_ICONS 8
typedef struct
{
    int            icon_length;
    icon_t         icon[PLAYER_MAX_ICONS];
    pos_t          last_pos;
    direction_t    dir;
    player_shoot_t shoot;
} player_t;


/* Create a new player (destroy with player_free_player) */
extern player_t *player_create_player(const map_t *map);


/* Release resources */
extern void player_free_player(player_t *player);


/* Move the player (auto play mode, game plays itself) */
extern game_status_t player_auto_player(player_t *player, const map_t *map);


/* Move the player (with keyboard) (human playing game) */
extern game_status_t player_move_player(player_t *player, const map_t *map);


/* Does the given position cross our player?
 * The segment that was hit is returned (0 is the head of the player)
 * -1 is returned if the position does not collide
 */
extern int player_check_collision(const player_t *player, const pos_t *pos);


/* Remove a piece of the player */
extern void player_loose_segment(player_t *player);


#endif /* _PLAYER_H */
