#include <ncurses.h>
#include "enemy.h"
#include "map.h"
#include "screen.h"


/* Set to avoid drawing the map over and over again.
 * screen_init() will reset this.
 */
static int screen_has_init_map;


void screen_init(void)
{
    initscr();
    noecho();
    cbreak();
    keypad(stdscr, TRUE);
    timeout(0);
    screen_has_init_map = 0;
}


void screen_exit(void)
{
    endwin();
}


void screen_refresh(void)
{
    refresh();
}


void screen_clear(void)
{
    clear();
}


void screen_render_map(const map_t *map)
{
    int i, j;

    if (screen_has_init_map)
      return;

    for (i=0; i<map->n_rows; ++i)
      for (j=0; j<map->n_cols; ++j)
      {
          move(i, j);
          printw("%c", map->map[i][j]);
      }

    screen_has_init_map = 1;
}


void screen_render_player(const player_t *player)
{
    int i;

    /* Draw icon */
    for (i=0; i<player->icon_length; ++i)
    {
        move(player->icon[i].pos.row, player->icon[i].pos.col);
        printw("%c", player->icon[i].symbol);
    }
}


void screen_render_enemies(const enemy_t *enemies)
{
    const enemy_t *ee;

    for (ee=enemies; ee; ee=ee->next)
    {
        move(ee->pos.row, ee->pos.col);
        printw("%c", ee->symbol);
    }
}


void screen_render_char(char c, const pos_t *pos)
{
    move(pos->row, pos->col);
    printw("%c", c);
}


void screen_render_status(
    const player_t *player,
    const map_t    *map,
    time_t          elapsed)
{
    /* Place the status bar under the map */
    move(map->n_rows, 0);
    printw("Elapsed Time:   %u seconds\n"
           "Remaining Time: %u seconds\n"
           "Length:         %d of %d\n",
           elapsed,
           TIMEOUT - elapsed,
           player->icon_length, PLAYER_MAX_ICONS);
}


void screen_display_welcome(void)
{
    printw("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"
           "D======================8   DILLDONGS  8=====================D\n"
           "D======================8      IN      8=====================D\n"
           "D======================8 OUTTER SPACE 8=====================D\n"
           "D======================8     v"VER"     8=====================D\n"
           "D======================8    enferex   8=====================D\n"
           "D======================8  757Labs.org 8=====================D\n"
           "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n");
    refresh();
    timeout(-1);
    getch();
    timeout(0);
}


void screen_display_results(time_t start_time, game_status_t status)
{
    clear();

    move(0, 0);
    printw("GAME OVER\n"
           "Time: %u seconds\n",
           (unsigned)(time(NULL) - start_time));

    switch (status)
    {
        case GAME_OVER_WIN:
          printw("Congratulations warrior, you have reached the "
                 "supreme length. Your handling skills are phenomenal, "
                 "and only the smoothest of operators have something "
                 "to fear. You are the envy of all men while the fantasy "
                 "of all women. Take caution as you brandish your weapon "
                 "with pride\n");
          break;

        case GAME_OVER:
          printw("Your handling skills are shameless and length reminiscient "
                 "of a baby carrot.\n");
          break;

        default:
          break;
    }

    timeout(-1);
    getch();
    timeout(0);
}


void screen_display_map_name(int level_num, const char *map_name)
{
    clear();

    move(0, 0);
    printw("Level: %d\n"
           "Map:   %s\n",
           level_num,
           map_name);

    timeout(-1);
    getch();
    timeout(0);
}
