#ifndef _SCREEN_H
#define _SCREEN_H

#include <ncurses.h>
#include <time.h>
#include "enemy.h"
#include "map.h"
#include "player.h"


/* Initalize the view, this will also force the map to
 * be redrawn.  So call this when loading a new map.
 */
extern void screen_init(void);


/* Clean up */
extern void screen_exit(void);


/* Refresh the window */
extern void screen_refresh(void);


/* Clear the screen */
extern void screen_clear(void);


/* Render map */
extern void screen_render_map(const map_t *map);


/* Render player */
extern void screen_render_player(const player_t *player);


/* Render enemies */
extern void screen_render_enemies(const enemy_t *enemies);


/* Render a char on the screen */
extern void screen_render_char(char c, const pos_t *pos);


/* Render the status of the game (elapsed in seconds) */
extern void screen_render_status(
    const player_t *player, 
    const map_t    *map,
    time_t          elapsed);


/* Welcome screen */
extern void screen_display_welcome(void);


/* End of game */
extern void screen_display_results(time_t start_time, game_status_t status);


/* Display the map name and level number */
extern void screen_display_map_name(int level_num, const char *map_name);


#endif /* _SCREEN_H */
